# Commitizen factory
NO_COMMITIZEN_FOUND = 1

# Config
NOT_A_GIT_PROJECT = 2
MISSING_CONFIG = 15

# Bump
NO_COMMITS_FOUND = 3
NO_VERSION_SPECIFIED = 4
NO_PATTERN_MAP = 5
COMMIT_FAILED = 6
TAG_FAILED = 7

# Commit
NO_ANSWERS = 8
COMMIT_ERROR = 9
NO_COMMIT_BACKUP = 10
NOTHING_TO_COMMIT = 11
CUSTOM_ERROR = 12

# Check
NO_COMMIT_MSG = 13
INVALID_COMMIT_MSG = 14

# Changelog
NO_REVISION = 16
